#!/bin/sh
# @(#) $Id: jpegrotate.sh 250 2010-04-11 23:39:40Z leres $ (LBL)
#
# jpegrotate - rotate jpeg images
#
# usage:
#
#       jpegrotate file ...    (auto-rotate using exitags)
#       jpegrotate -l file ... (rotate left)
#       jpegrotate -r file ... (rotate right)
#       jpegrotate -f file ... (rotate 180)
#       jpegrotate -h file ... (horizontal)
#       jpegrotate -v file ... (vertical)
#       jpegrotate -u file ... (undo)
#

prog="`basename $0`"
usage="usage: ${prog} [-n] (-l|-r|-v|-h|-f|-u) file ..."

dryrun=0
option=""
status=0
undo=0

set -- `getopt fhlnruv $*`

for a do
	case "${a}" in

	-l)
		option="-rotate 270"
		shift
		;;

	-r)
		option="-rotate 90"
		shift
		;;

	-f)
		option="-rotate 180"
		shift
		;;

	-h)
		option="-flip horizontal"
		shift
		;;

	-n)
		dryrun=1
		shift
		;;

	-v)
		option="-flip vertical"
		shift
		;;

	-u)
		undo=1
		shift
		;;

	--)
		shift
		break
		;;
	esac
done

if [ -n "${option}" -a ${undo} -ne 0 ]; then
	echo "${usage}" 1>&2
	exit 1
fi

if [ $# -lt 1 ]; then
	echo "${usage}" 1>&2
	exit 1
fi

# We need the sysv version under Solaris
touch="/usr/bin/touch"
if [ ! -x ${touch} ]; then
	touch="touch"
fi

# Execute (or echo if -n)
execute() {
	cmd="$*"
	if [ ${dryrun} -eq 0 ]; then
		${cmd}
		return $?
	fi
	echo "+ ${cmd}"
	return 0
}


if [ ${undo} -ne 0 ]; then
	# Undo logic
	while [ $# -gt 0 ]; do
		r="$1"
		shift

		f="`echo ${r} |
		    sed -e 's/r\.jpg$/.jpg/' -e 's/r\.JPG$/.JPG/'`"
		if [ "${f}" = "${r}" ]; then
			echo "${prog}: ${r} already unrotated? (skipping)"
			status=1
			continue
		fi
		if [ -r ${r} -a -r ${f} ]; then
			echo "${prog}: Both ${r} and ${f} exist (skipping)"
			status=1
			continue
		fi
		if [ -r ${f} ]; then
			echo "${prog}: ${f} already exists (skipping)"
			status=1
			continue
		fi
		if [ ! -r ${r} ]; then
			echo "${prog}: Can't find ${r}"
			status=1
			continue
		fi
		o="${f}-"
		if [ ! -r ${o} ]; then
			echo "${prog}: ${o} missing (skipping)"
			status=1
			continue
		fi

		execute "rm -f ${r}-"
		execute "mv ${r} ${r}-"
		execute "mv ${o} ${f}"
		echo "${prog}: ${r} -> ${f}"
	done
	exit ${status}
fi

auto=0
if [ -z "${option}" ]; then
	auto=1
	# Make sure we have everything we need
	for f in exiftags jhead jpegtran; do
		which ${f} > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "${prog}: Can't find ${f}" 1>&2
			exit 1
		fi
	done
fi

# Rotate logic
while [ $# -gt 0 ]; do
	f="$1"
	shift

	r="`echo ${f} | sed -e 's/r*\(\.[Jj][Pp][Gg]\)$/r\1/'`"
	if [ "${f}" = "${r}" ]; then
                echo "${prog}: ${f} already rotated? (skipping)"
		status=1
                continue
	fi
        if [ -r ${r} ]; then
                echo "${prog}: ${r} already exists, skipping ${f}"
		status=1
                continue
	fi
        if [ ! -r ${f} ]; then
                echo "${prog}: Can't find ${f}"
		status=1
                continue
        fi
	o="${f}-"
        if [ -r ${o} ]; then
                echo "${prog}: ${o} already exists (skipping)"
		status=1
                continue
        fi

	# Auto-detect
	if [ ${auto} -ne 0 ]; then
		rotate=`exiftags ${f} 2>&1 |
		    awk '
		    /Image Orientation: Top, Left-Hand/ {
			    print "0"
			    next
		    }
		    /Image Orientation: Right-Hand, Top/ {
			    print "90"
			    next
		    }
		    /Image Orientation: Left-Hand, Bottom/ {
			    print "270"
			    next
		    }
		    /Image Orientation: / {
			    i = index($0, ": ")
			    if (i > 0)
				    print "# " substr($0, i + 2)
			    next
		    }
		    '`
		case "${rotate}" in

		0)
			# No transformation needed
			continue
			;;

		[0-9]*)
			# Ok!
			;;

		*)
			echo "${prog}: Can't parse ${f} (${rotate})" 1>&2
			exit 1
			;;
		esac

		option="-rotate ${rotate}"
	fi

	# Pick up permissions from original file
	execute "cp -p ${f} ${r}"
	if [ $? -ne 0 ]; then
		exit 1
	fi
	execute "chmod +w ${r}"

	# Transform
	#jpegtran -copy all ${option} ${f} > ${r}
	execute "jhead -q -autorot ${r}"
	if [ $? -ne 0 ]; then
		rm ${r}
		exit 1
	fi

	# Do this after the jpegtran in case jpegtran fails
	execute "mv ${f} ${o}"

	# Copy access and modified timestamps
	execute "${touch} -r ${o} ${r}"
	echo "${prog}: ${f} -> ${r}"
done

exit ${status}
